; ADPCM-B playback demo (68K code) by freem
; assemble with vasm m68k (motorola syntax)

; quick and dirty shell prog

; this is not representative of a properly fully working neo-geo program,
; it only implements enough stuff to get the job done.

; don't ask me to port this to Neo-Geo CD, because NGCD has no ADPCM-B.
;==============================================================================;
	include "inc/neogeo.inc"			; hardware locations
	include "inc/ram_bios.inc"			; BIOS RAM locations
	include	"inc/input.inc"				; input defines
	include "ram_user.inc"				; program RAM locations
;------------------------------------------------------------------------------;
	; headers
	include "header_68k.inc"
	include "header_cart.inc"
;==============================================================================;
; USER
; Needs to perform actions according to the value in BIOS_USER_REQUEST.
; Must jump back to SYSTEM_RETURN at the end so the BIOS can have control.

USER:
	move.b	d0,REG_DIPSW		; kick watchdog
	lea		BIOS_WORKRAM,sp		; set stack pointer to BIOS_WORKRAM
	move.w	#0,LSPC_MODE		; Disable auto-animation, timer interrupts, set auto-anim speed to 0 frames
	move.w	#7,LSPC_IRQ_ACK		; ack. all IRQs

	move.w	#$2000,sr			; Enable VBlank interrupt, go Supervisor

	; Handle user request
	clr.l	d0
	move.b	(BIOS_USER_REQUEST).l,d0
	lsl.b	#2,d0				; shift value left to get offset into table
	lea		cmds_USER_REQUEST,a0
	movea.l	(a0,d0),a0
	jsr		(a0)

;------------------------------------------------------------------------------;
; BIOS_USER_REQUEST commands
cmds_USER_REQUEST:
	dc.l	userReq_StartupInit	; Command 0 (Initialize)
	dc.l	SYSTEM_RETURN		; Command 1 (Custom eyecatch)
	dc.l	userReq_Game		; Command 2 (Demo Game/Game)
	dc.l	userReq_Game		; Command 3 (Title Display)

;------------------------------------------------------------------------------;
; userReq_StartupInit
; Initialize the backup work area.

userReq_StartupInit:
	move.b	d0,REG_DIPSW		; kick watchdog
	jmp		SYSTEM_RETURN

;------------------------------------------------------------------------------;
; userReq_Game
; This is the complex one.

userReq_Game:
	move.b	d0,REG_DIPSW		; kick watchdog

	; perform initialization
	lea		LSPC_DATA,a5		; LSPC data reg in A5 so I can abuse it

	move.b	d0,SYS_CARTVEC
	move.b	d0,SYS_CARTFIX

	move.b	#3,sampleRateNum	; start at 44100
	move.w	#44100,bin_SampleRate	; initial sample rate (binary; calcs)
	move.w	#$CB6B,deltaNValue	; initial delta-n

	move.b	#$C0,flag_OutputLR	; initial Left/Right output

	move.b	#$F,outputLevel		; initial output level
	move.b	#$FF,volumeLevel	; initial volume level

	clr.b	curItemNum			; first item
	clr.b	prevItemNum			; prev item

	move.b	#3,REG_SOUND

	; set up palettes
	move.b	d0,PALETTE_BANK1	; use palette bank 1
	lea		paletteData,a0
	lea		PALETTES,a1
	move.l	#(16*NUM_PALETTES)-1,d7
.ldpal:
	move.w	(a0)+,(a1)+
	dbra	d7,.ldpal

	jsr		FIX_CLEAR			; clear fix layer, add borders on sides
	jsr		LSP_1st				; clear first sprite

	jsr		InitDisplay

; execution continues into main loop.
;------------------------------------------------------------------------------;
; mainLoop
; The game's main loop. This is where you run the actual game part.

mainLoop:
	move.b	d0,REG_DIPSW		; kick the watchdog

	; wait for vblank
	jsr		WaitVBlank

	; after vblank
	jsr		CheckButtons		; Check button input
	jsr		CheckJoy			; Check joystick input

	; loop back to top
	jmp		mainLoop

;==============================================================================;
; PLAYER_START
; This demo doesn't care about coins, what are you talking about?

PLAYER_START:
	move.b	d0,REG_DIPSW		; kick the watchdog
	rts

;==============================================================================;
; DEMO_END
; we don't care about storing things in MVS backup RAM.

DEMO_END:
	rts

;==============================================================================;
; COIN_SOUND
; Called by the BIOS when a coin is inserted; should play a coin drop sound.

COIN_SOUND:
	; Send a sound code
	rts

;==============================================================================;
; VBlank
VBlank:
	; check if the BIOS wants to run its vblank
	btst	#7,BIOS_SYSTEM_MODE
	bne		.gamevbl
	; run BIOS vblank
	jmp		SYSTEM_INT1

	; run the game's vblank
.gamevbl
	movem.l d0-d7/a0-a6,-(sp)	; save registers
	move.w	#4,LSPC_IRQ_ACK		; acknowledge the vblank interrupt
	move.b	d0,REG_DIPSW		; kick the watchdog

	; this is where you would do your own things in vblank, but we don't here.

.endvbl:
	jsr		SYSTEM_IO			; "Call SYSTEM_IO every 1/60 second."
	jsr		MESS_OUT			; Puzzle Bobble calls MESS_OUT just after SYSTEM_IO
	move.b	#0,flag_VBlank		; clear vblank flag so waitVBlank knows to stop
	movem.l (sp)+,d0-d7/a0-a6	; restore registers
	rte

;==============================================================================;
; IRQ2
; Level 2/timer interrupt, unused here. You could use it for effects, though.

IRQ2:
	move.w	#2,LSPC_IRQ_ACK		; ack. interrupt #2 (HBlank)
	move.b	d0,REG_DIPSW		; kick watchdog

	rte

;==============================================================================;
; WaitVBlank
; Waits for VBlank to finish (via a flag cleared at the end).

WaitVBlank:
	move.b	#1,flag_VBlank

.waitLoop
	tst.b	flag_VBlank
	bne.s	.waitLoop

	rts

;==============================================================================;
; freemlib includes
	include "freemlib/fix.asm"
;==============================================================================;
; program data
	include "paldata.inc"
;==============================================================================;
; header
str_Header:			dc.b "ADPCM-B TEST",$FF,$00
str_Feature:		dc.b "FEATURING WFL ACTION POINT",$FF,$00

; sample rate
str_SampleRate:		dc.b "SAMPLE RATE:",$FF,$00

; delta n
str_DeltaN:			dc.b "DELTA-N:",$FF,$00
str_InitDeltaN:		dc.b "CB6B",$FF,$00

; channels
str_Channels:		dc.b "CHANNELS:",$FF
str_Left:			dc.b "L  ",$FF
str_Right:			dc.b "R  ",$FF
str_Both:			dc.b "L/R",$FF

; volume
str_Volume:			dc.b "VOLUME:",$FF
str_InitVolume:		dc.b "$FF",$FF

; button instructions
str_Play:			dc.b $25," PLAY SAMPLE",$FF
str_Stop:			dc.b $26," STOP PLAYING",$FF,$00
str_Prev:			dc.b $12," PREV. ITEM",$FF,$00
str_Next:			dc.b $13," NEXT ITEM",$FF

;------------------------------------------------------------------------------;
; InitDisplay
; Draw the initial display (Fix layer stuff)

InitDisplay:
	; header
	move.w	#$0E04,d0
	move.b	#$03,d1
	lea		str_Header,a0
	jsr		fix_DrawString

	move.w	#$0705,d0
	move.b	#$03,d1
	lea		str_Feature,a0
	jsr		fix_DrawString

	; draw initial Main Cursor
	move.w	#$704A,-2(a5)		; lspc address
	move.w	#$0311,0(a5)		; lspc data

	; sample rate label
	move.w	#$030A,d0
	move.b	#$03,d1
	lea		str_SampleRate,a0
	jsr		fix_DrawString

	; initial sample rate text
	move.w	#$100A,d0
	move.b	#$03,d1
	lea		str_SampRate_44100,a0
	jsr		fix_DrawString

	; delta-n label
	move.w	#$180A,d0
	move.b	#$03,d1
	lea		str_DeltaN,a0
	jsr		fix_DrawString

	; initial delta-n value text
	move.w	#$210A,d0
	move.b	#$03,d1
	lea		str_InitDeltaN,a0
	jsr		fix_DrawString

	; channels
	move.w	#$030D,d0
	move.b	#$03,d1
	lea		str_Channels,a0
	jsr		fix_DrawString

	; initial channel setting text
	move.w	#$100D,d0
	move.b	#$03,d1
	lea		str_Both,a0
	jsr		fix_DrawString

	; volume
	move.w	#$030F,d0
	move.b	#$03,d1
	lea		str_Volume,a0
	jsr		fix_DrawString

	; initial volume text
	move.w	#$0F0F,d0
	move.b	#$03,d1
	lea		str_InitVolume,a0
	jsr		fix_DrawString

	; instructions
	move.w	#$0C16,d0
	move.b	#$03,d1
	lea		str_Play,a0
	jsr		fix_DrawString

	move.w	#$0C17,d0
	move.b	#$03,d1
	lea		str_Stop,a0
	jsr		fix_DrawString

	move.w	#$0C18,d0
	move.b	#$03,d1
	lea		str_Prev,a0
	jsr		fix_DrawString

	move.w	#$0C19,d0
	move.b	#$03,d1
	lea		str_Next,a0
	jsr		fix_DrawString

	rts

;==============================================================================;
; CheckJoy
; Check for (and handle) joystick input.

CheckJoy:
	move.b	BIOS_P1CHANGE,d0
	move.b	d0,d1
	andi.b	#INPUT_UDLR,d0
	beq		.CheckJoy_end		; no directions

	; a direction was pressed.
	; was it up/down?
	andi.b	#INPUT_UD,d0
	beq		.CheckJoy_LR

	; it was. whew.
	move.b	d1,d0
	andi.b	#INPUT_UP,d0
	beq		.CheckJoy_Down

	; update previous item number
	move.b	curItemNum,d2
	move.b	d2,prevItemNum

	; pressed up, go to previous item
	move.b	curItemNum,d2
	bne		.CheckJoy_PrevItem

	; move to last item
	move.b	#2,curItemNum
	jsr		UpdateMainCursor
	bra		.CheckJoy_end

.CheckJoy_PrevItem:
	subi.b	#1,curItemNum
	jsr		UpdateMainCursor
	bra		.CheckJoy_end

.CheckJoy_Down:
	move.b	d1,d0
	andi.b	#INPUT_DOWN,d0
	beq		.CheckJoy_end

	; update previous item number
	move.b	curItemNum,d2
	move.b	d2,prevItemNum

	; pressed down, go to next item
	cmpi.b	#2,curItemNum
	bne		.CheckJoy_NextItem

	; move to first item
	clr.b	curItemNum
	jsr		UpdateMainCursor
	bra		.CheckJoy_end

.CheckJoy_NextItem:
	addi.b	#1,curItemNum
	jsr		UpdateMainCursor
	bra		.CheckJoy_end

.CheckJoy_LR:
	; In order to see what we need to do, check curItemNum.
	cmpi.b	#2,curItemNum
	beq		CheckJoy_Volume

	cmpi.b	#1,curItemNum
	beq		CheckJoy_Channels

	move.b	curItemNum,d0
	beq		CheckJoy_SampleRate

.CheckJoy_end:
	rts

;------------------------------------------------------------------------------;
; CheckJoy_SampleRate

; Left/Right: Change sample rate

CheckJoy_SampleRate:
	move.b	d1,d0
	andi.b	#INPUT_LR,d0
	beq		.CheckJoy_SampleRate_end

	; pressed left or right. which one?
	andi.b	#INPUT_LEFT,d0
	beq		.CheckJoy_SampleRate_Right

	; moved left (sample rate down)
	tst.b	sampleRateNum
	bne		.CheckJoy_SampleRate_MoveLeft

	; wrap around
	move.b	#4,sampleRateNum
	jsr		UpdateRateText		;update sample number text
	jsr		UpdateDeltaN		;update delta-n value
	jsr		WriteRateValue		;send command
	bra		.CheckJoy_SampleRate_end

.CheckJoy_SampleRate_MoveLeft:
	subi.b	#1,sampleRateNum
	jsr		UpdateRateText		;update sample number text
	jsr		UpdateDeltaN		;update delta-n value
	jsr		WriteRateValue		;send command
	bra		.CheckJoy_SampleRate_end

.CheckJoy_SampleRate_Right:
	; moved right (sample rate up)
	;check for max
	move.b	sampleRateNum,d0
	cmpi.b	#4,d0
	bne	.CheckJoy_SampleRate_MoveRight

	; wrap around
	clr.b	sampleRateNum
	jsr		UpdateRateText		;update sample number text
	jsr		UpdateDeltaN		;update delta-n value
	jsr		WriteRateValue		;send command
	bra		.CheckJoy_SampleRate_end

.CheckJoy_SampleRate_MoveRight:
	addi.b	#1,sampleRateNum
	jsr		UpdateRateText		;update sample number text
	jsr		UpdateDeltaN		;update delta-n value
	jsr		WriteRateValue		;send command
	bra		.CheckJoy_SampleRate_end

.CheckJoy_SampleRate_end:
	rts

;------------------------------------------------------------------------------;
; CheckJoy_Channels

; Left/Right: Cycle through output L/R/LR

CheckJoy_Channels:
	move.b	d1,d0
	andi.b	#INPUT_LR,d0
	beq		.CheckJoy_Channels_end	; what are you even doing here?

	andi.b	#INPUT_LEFT,d0
	beq		.CheckJoy_Channels_Right

	; pressed left
	move.b	flag_OutputLR,d0
	cmpi.b	#$C0,d0					; both
	bne		.CheckJoy_Channels_Left2
	move.b	#$40,flag_OutputLR
	bra		.CheckJoy_Channels_end

.CheckJoy_Channels_Left2:
	cmpi.b	#$80,d0					; left only
	bne		.CheckJoy_Channels_Left3
	move.b	#$C0,flag_OutputLR
	bra		.CheckJoy_Channels_end

.CheckJoy_Channels_Left3:
	move.b	#$80,flag_OutputLR
	bra		.CheckJoy_Channels_end

.CheckJoy_Channels_Right:
	; pressed right
	move.b	flag_OutputLR,d0
	cmpi.b	#$C0,d0					; both
	bne		.CheckJoy_Channels_Right2
	move.b	#$80,flag_OutputLR
	bra		.CheckJoy_Channels_end

.CheckJoy_Channels_Right2:
	cmpi.b	#$80,d0					; left only
	bne		.CheckJoy_Channels_Right3
	move.b	#$40,flag_OutputLR
	bra		.CheckJoy_Channels_end

.CheckJoy_Channels_Right3:
	move.b	#$C0,flag_OutputLR

.CheckJoy_Channels_end:
	jsr		UpdateChannels
	rts

;------------------------------------------------------------------------------;
; CheckJoy_Volume

; Left/Right: Change volume

CheckJoy_Volume:
	move.b	d1,d0
	andi.b	#INPUT_LR,d0
	beq		.CheckJoy_Volume_end
	; pressed Left or Right... which one?

	andi.b	#INPUT_LEFT,d0
	beq		.CheckJoy_Volume_Right

	; pressed left
	tst.b	outputLevel
	bne		.CheckJoy_Volume_Previous

	; wrap around
	move.b	#$F,outputLevel
	jsr		UpdateVolumeText
	jsr		WriteVolumeValue
	bra		.CheckJoy_Volume_end

.CheckJoy_Volume_Previous:
	subi.b	#1,outputLevel
	jsr		UpdateVolumeText
	jsr		WriteVolumeValue
	bra		.CheckJoy_Volume_end

.CheckJoy_Volume_Right:
	move.b	d1,d0
	andi.b	#INPUT_RIGHT,d0
	beq		.CheckJoy_Volume_end

	; pressed right
	move.b	outputLevel,d0
	cmpi.b	#$F,d0
	bne		.CheckJoy_Volume_Next

	; wrap around
	clr.b	outputLevel
	jsr		UpdateVolumeText
	jsr		WriteVolumeValue
	bra		.CheckJoy_Volume_end

.CheckJoy_Volume_Next:
	addi.b	#1,outputLevel
	jsr		UpdateVolumeText
	jsr		WriteVolumeValue

.CheckJoy_Volume_end:
	rts

;==============================================================================;
; CheckButtons
; Check for (and handle) button input.

; A		Play sample (Command $20)
; B		Stop sample (Command $0D)

CheckButtons:
	move.b	BIOS_P1CHANGE,d0
	move.b	d0,d1				; make pre-mask copy
	andi.b	#INPUT_AB,d0		; mask for buttons
	beq		.CheckButtons_end	; if not found, CheckButtons_end

.CheckButtons_A:
	move.b	d1,d0
	andi.b	#INPUT_A,d0			; check for A
	beq		.CheckButtons_B		; not pressed, check for B.

	; pressed A (play sample)
	move.b	#$20,REG_SOUND		; play sound $20
	bra		.CheckButtons_end	; don't check for B, even if it was pressed?

.CheckButtons_B:
	move.b	d1,d0
	andi.b	#INPUT_B,d0			; check for B
	beq		.CheckButtons_end	; not pressed, we're done.

	; pressed B (stop sample)
	move.b	#$0D,REG_SOUND		; write $0D (stop sound)

.CheckButtons_end:
	rts

;==============================================================================;
; main cursor locations
tbl_MainCursorLoc:
	dc.w	$704A
	dc.w	$704D
	dc.w	$704F

;------------------------------------------------------------------------------;
; UpdateMainCursor
; Update the location of the main cursor.

UpdateMainCursor:
	; erase cursor where previous item was
	moveq	#0,d0
	moveq	#0,d1
	move.b	prevItemNum,d0
	add.w	d0,d0
	move.w	tbl_MainCursorLoc(pc,d0.w),d1
	move.w	d1,-2(a5)			; set address
	move.w	#$00FF,0(a5)

	; draw cursor where new item is
	moveq	#0,d0
	moveq	#0,d1
	move.b	curItemNum,d0
	add.w	d0,d0
	move.w	tbl_MainCursorLoc(pc,d0.w),d1
	move.w	d1,-2(a5)			; set address
	move.w	#$0311,0(a5)

	rts

;==============================================================================;
; UpdateChannels
; Updates the channel text and sends the relevant sound command.

UpdateChannels:
	; determine what string belongs in a0
	lea		str_Both,a0
	move.b	#$42,d4				; sound command for left/right output
	move.b	flag_OutputLR,d0
	cmpi.b	#$C0,d0
	beq		.UpdateChannels_draw

	lea		str_Left,a0
	move.b	#$40,d4				; sound command for left output
	cmpi.b	#$80,d0
	beq		.UpdateChannels_draw

	lea		str_Right,a0
	move.b	#$41,d4				; sound command for right output

.UpdateChannels_draw:
	; draw string
	move.w	#$100D,d0
	move.b	#$03,d1
	jsr		fix_DrawString
	move.b	d4,REG_SOUND		; send sound command

.UpdateChannels_end:
	rts

;==============================================================================;
; UpdateVolumeText
; Update the volume value text.

UpdateVolumeText:
	move.b	outputLevel,d0
	cmpi.b	#$F,d0
	bne		.UpdateVolumeText_Normal

	; special case $FF
	move.b	#$FF,volumeLevel
	bra		.UpdateVolumeText_Draw

.UpdateVolumeText_Normal:
	; not $FF
	mulu.b	#$10,d0
	move.b	d0,volumeLevel

.UpdateVolumeText_Draw:
	; upper digit
	moveq	#0,d0
	move.b	volumeLevel,d0
	andi.b	#$F0,d0
	lsr.b	#4,d0
	cmpi.b	#$0A,d0
	bge		.UpdateVolumeText_UpperHex

	; upper digit 0-9
	addi.w	#$0330,d0
	bra		.UpdateVolumeText_DrawUpper

.UpdateVolumeText_UpperHex:
	; upper digit A-F
	addi.w	#$0337,d0

.UpdateVolumeText_DrawUpper:
	move.w	#$720F,-2(a5)
	move.w	d0,0(a5)

.UpdateVolumeText_Lower:
	; lower digit
	moveq	#0,d0
	move.b	volumeLevel,d0
	andi.b	#$0F,d0
	cmpi.b	#$0A,d0
	bge		.UpdateVolumeText_LowerHex

	; lower digit 0-9
	addi.w	#$0330,d0
	bra		.UpdateVolumeText_DrawLower

.UpdateVolumeText_LowerHex:
	; lower digit A-F
	addi.w	#$0337,d0

.UpdateVolumeText_DrawLower:
	move.w	#$722F,-2(a5)
	move.w	d0,0(a5)

.UpdateVolumeText_end:
	rts

;==============================================================================;
tbl_volumeCommands:
	dc.b $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F

;------------------------------------------------------------------------------;
; WriteVolumeValue
; Sends the volume command to the sound driver.

WriteVolumeValue:
	moveq	#0,d0				; clear this because I don't trust people
	move.b	outputLevel,d0		; use outputLevel as index into tbl_volumeCommands

	; send volume command (yes, I should probably just use outputLevel+$80 instead)
	move.b	tbl_volumeCommands(pc,d0.w),REG_SOUND
	rts

;==============================================================================;
tbl_SampRateStrings:
	dc.l	str_SampRate_11025
	dc.l	str_SampRate_22050
	dc.l	str_SampRate_33075
	dc.l	str_SampRate_44100
	dc.l	str_SampRate_55500

str_SampRate_11025:	dc.b "11025",$15,$FF,$00
str_SampRate_22050:	dc.b "22050",$15,$FF,$00
str_SampRate_33075:	dc.b "33075",$15,$FF,$00
str_SampRate_44100:	dc.b "44100",$15,$FF,$00
str_SampRate_55500:	dc.b "55500",$15,$FF,$00

tbl_SampRateBin:
	dc.w	$2B11
	dc.w	$5622
	dc.w	$8133
	dc.w	$AC44
	dc.w	$DBCC

;------------------------------------------------------------------------------;
; UpdateRateText
; Updates the sample rate text.

UpdateRateText:
	moveq	#0,d0

	; update bin_SampleRate for Delta-N calculations
	move.b	sampleRateNum,d0
	add.w	d0,d0
	move.w	tbl_SampRateBin(pc,d0.w),bin_SampleRate

	; update display string
	moveq	#0,d0
	moveq	#0,d1
	move.b	sampleRateNum,d0
	lsl.w	#2,d0
	movea.l	tbl_SampRateStrings(pc,d0.w),a1
	lea		(a1),a0
	move.w	#$100A,d0
	move.b	#$03,d1
	jsr		fix_DrawString

	rts

;==============================================================================;
tbl_DeltaNVals:
	dc.w	$32DB
	dc.w	$65B5
	dc.w	$9890
	dc.w	$CB6B
	dc.w	$FFFF

tbl_DeltaNStrings:
	dc.l	str_DeltaN_0
	dc.l	str_DeltaN_1
	dc.l	str_DeltaN_2
	dc.l	str_DeltaN_3
	dc.l	str_DeltaN_4

str_DeltaN_0: dc.b	"32DB",$FF,$00
str_DeltaN_1: dc.b	"65B5",$FF,$00
str_DeltaN_2: dc.b	"9890",$FF,$00
str_DeltaN_3: dc.b	"CB6B",$FF,$00
str_DeltaN_4: dc.b	"FFFF",$FF,$00

;------------------------------------------------------------------------------;
; UpdateDeltaN
; Display the hardcoded delta-N values used in the sound driver.

UpdateDeltaN:
	moveq	#0,d0				; clear! *bzzzzzt*

	; use sampleRateNum as index into X
	move.b	sampleRateNum,d0
	add.w	d0,d0
	; update deltaNValue
	move.w	tbl_DeltaNVals(pc,d0.w),deltaNValue

	; update display text
	moveq	#0,d0
	moveq	#0,d1
	move.b	sampleRateNum,d0
	lsl.w	#2,d0
	movea.l	tbl_DeltaNStrings(pc,d0.w),a1
	lea		(a1),a0
	move.w	#$210A,d0
	move.b	#$03,d1
	jsr		fix_DrawString

	rts

;==============================================================================;
tbl_RateCommands:
	dc.b	$C0,$C1,$C2,$C3,$C4,$00

;------------------------------------------------------------------------------;
; WriteRateValue
; Sends the rate command to the sound driver.

WriteRateValue:
	moveq	#0,d0				; clear this because I don't trust people
	move.b	sampleRateNum,d0	; use sampleRateNum as index into tbl_RateCommands

	; send rate command (yes, I should probably just use sampleRateNum+$C0 instead)
	move.b	tbl_RateCommands(pc,d0.w),REG_SOUND
	rts
