; ADPCM-B playback demo (Z80 code) by freem
; assemble with vasm z80 (oldstyle syntax)

; This driver probably does a lot more than it needs to... that's because I'm
; using the work-in-progress freemlib M1 driver as a base for this one.
;==============================================================================;
	include "sounddef.inc"
	include "sysmacro.inc"
;==============================================================================;
	org $0000
; $0000: Disable interrupts and jump to the real entry point
Start:
	di
	jp		EntryPoint
;-------------------------------------------------------------------------------
	org $0008
; Port Delay Write for Addresses
portWriteDelayPart1:
	jp		portWriteDelayPart2
;------------------------------------------------------------------------------;
; xxx: for some reason, the driver gets sent here on MVS.
	org $000C
	rst 0
;------------------------------------------------------------------------------;
	org $0010
; Port Delay Write for Data
portWriteDelayPart3:
	jp		portWriteDelayPart4
;------------------------------------------------------------------------------;
	org $0018
j_write45:
	jp		write_45
;------------------------------------------------------------------------------;
	org $0020
j_write67:
	jp		write_67
;------------------------------------------------------------------------------;
	org $0028
; Keep checking the busy flag in Status 0 until it's clear.

; Code from smkdan's example M1 driver (adpcma_demo2/sound_M1.asm), where he
; uses this instead of portWriteDelayPart2 and portWriteDelayPart4.

CheckBusyFlag:
	in		a,(4)			; read Status 0 (busy flag in bit 7)
	add		a
	jr		c,CheckBusyFlag
	ret
;------------------------------------------------------------------------------;
	org $0030
	ret
;------------------------------------------------------------------------------;
	org $0038
; the IRQ belongs here.
j_IRQ:
	di
	jp		IRQ
;==============================================================================;
	org $0040
; driver signature
driverSig:
	asc "adpcm-b playback demo M1 driver [f]"
;==============================================================================;
	org $0066
; NMI
; Inter-processor communications.

; In this driver, the NMI gets the command from the 68K and interprets it.

NMI:
	; save registers
	push	af
	push	bc
	push	de
	push	hl

	in		a,(0)			; Acknowledge NMI, get command from 68K via Port 0
	ld		(curCommand),a	; update curCommand
	ld		b,a				; load value into b for comparisons

	; "Commands $01 and $03 are always expected to be implemented as they
	; are used by the BIOSes for initialization purposes." - NeoGeoDev Wiki
	cp		#1				; Command 1 (Slot Switch)
	jp		Z,doCmd01
	cp		#3				; Command 3 (Soft Reset)
	jp		Z,doCmd03
	or		a				; check if Command is 0
	jp		Z,endNMI		; exit if Command 0

	; do NMI crap/handle communication... I'm not fully sure what to do yet.
	call	HandleCommand

	xor		a				; clear a for now.
	out		(0xC),a			; Reply to 68K with something.
	out		(0),a			; Write to port 0 (Clear sound code)

	; update previous command
	ld		a,(curCommand)
	ld		(prevCommand),a
endNMI:
	; restore registers
	pop		hl
	pop		de
	pop		bc
	pop		af
	retn

;==============================================================================;
; IRQ (called from $0038)
; Handle an interrupt request.

; In this driver, the IRQ is used for keeping the music playing.
; At least, that's the goal. Not sure how feasible it really is.

; Some other engines use the IRQ to poll the two status ports (6 and 4).
; IRQs in SNK drivers (Mr.Pac, MAKOTO v3.0) are pretty large. I want to avoid
; that, if at all possible.

IRQ:
	; save registers
	push	af
	push	bc
	push	de
	push	hl
	push	ix
	push	iy

	; update internal Status 1 register
	in		a, (6)
	ld		(intStatus1),a

	; check status of ADPCM channels
	;ld		a,(intStatus1)
	;bit		7,a
	;call	NZ,pcmb_Silence

	; update internal Status 0 register
	in		a, (4)
	ld		(intStatus0),a

endIRQ:
	; restore registers
	pop		iy
	pop		ix
	pop		hl
	pop		de
	pop		bc
	pop		af

	ei

	; return
	ret						; was "reti", see note below.

; note:
; "In an IRQ, RETI is only useful if you have something like a Z80 PIO to support
; daisy-chaining: queuing interrupts. The PIO can detect that the routine has
; ended by the opcode of RETI, and let another device generate an interrupt."

;==============================================================================;
; EntryPoint
; The entry point for the sound driver.

EntryPoint:
	ld		sp,0xFFFC		; Set stack pointer ($FFFD-$FFFE is used for other purposes)
	im		1				; Set Interrupt Mode 1 (IRQ at $38)
	xor		a				; make value in A = 0

	; Clear RAM at $F800-$FFFF
	ld		(0xF800),a		; set $F800 = 0
	ld		hl,0xF800		; 00 value is at $F800
	ld		de,0xF801		; write sequence begins at $F801
	ld		bc,0x7FF		; end at $FFFF
	ldir					; clear out memory

	; Initialize variables

	; Silence SSG, FM(, and ADPCM?)
	call	ssg_Silence
	call	fm_Silence
	; "various writes to ports 4/5 and 6/7"

	;-------------------------------------------;
	; write 1 to port $C0 (what is the purpose?)
	ld		a,1
	out		(0xC0),a

	; continue setting up the hardware, etc.

	ld		de,0x2730		; Reset Timer flags, Disable Timer IRQs
	write45					; write to ports 4 and 5
	ld		de,0x1001		; Reset ADPCM-B
	write45					; write to ports 4 and 5
	ld		de,0x1C00		; Unmask ADPCM-A and B flag controls
	write45					; write to ports 4 and 5

	; Initialize more variables
	call	SetDefaultBanks	; Set default program banks

; this section subject to further review
;{
	; set timer values??

	; Start Timers ($27,$3F to ports 4 and 5)
	ld		de,0x273F		; Reset Timer flags, Enable Timer IRQs, Load Timers
	write45					; write to ports 4 and 5

	; (ADPCM-A shared volume)
	ld		de,0x013F		; Set ADPCM-A volume to Maximum
	write67					; write to ports 6 and 7
;}

	; Perform program-specific ADPCM-B initialization
	ld		a,0xC0
	ld		(intPCMB_LeftRight),a	; ADPCM-B left and right channels
	ld		de,0x11C0		; Set ADPCM-B Left/Right output
	write45					; write to ports 4 and 5

	ld		a,0x6B
	ld		(intPCMB_SampRateL),a	; Delta-N Sampling Rate (L)
	ld		de,0x196B		; Set ADPCM-B Sampling Rate LSB
	write45					; write to ports 4 and 5

	ld		a,0xCB
	ld		(intPCMB_SampRateH),a	; Delta-N Sampling Rate (H)
	ld		de,0x1ACB		; Set ADPCM-B Sampling Rate LSB
	write45					; write to ports 4 and 5

	ld		a,0xFF
	ld		(intPCMB_Volume),a		; ADPCM-B volume max
	ld		de,0x1BFF		; Set ADPCM-B volume to Maximum
	write45					; write to ports 4 and 5

	; (Enable NMIs)
	ld		a,1
	out		(8),a			; Write to Port 8 (Enable NMI)
	ei

;------------------------------------------------------------------------------;
; MainLoop
; Nothing here

MainLoop:
	;ei
	jp		MainLoop

;==============================================================================;
; write_45
; Writes data (from registers de) to ports 4 and 5. (YM2610 A1 line=0)

write_45:
	push	af
	ld		a,d
	out		(4),a			; write to port 4 (address 1)
	rst		8				; Write delay 1 (17 cycles)
	ld		a,e
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	pop		af
	ret

;------------------------------------------------------------------------------;
; write_67
; Writes data (from registers de) to ports 6 and 7. (YM2610 A1 line=1)

write_67:
	push	af
	ld		a,d
	out		(6),a			; write to port 6 (address 2)
	rst		8				; Write delay 1 (17 cycles)
	ld		a,e
	out		(7),a			; write to port 7 (data 2)
	rst		0x10			; Write delay 2 (83 cycles)
	pop		af
	ret

;------------------------------------------------------------------------------;
; portWriteDelayPart2
; Part 2 of the write delay for ports (address port 2/2).
; (burns 17 cycles on YM2610)

portWriteDelayPart2:
	ret

;------------------------------------------------------------------------------;
; portWriteDelayPart4
; Part 4 of the write delay for ports (data port 2/2).
; (burns 83 cycles on YM2610)

portWriteDelayPart4:
	push	bc
	push	de
	push	hl
	pop		hl
	pop		de
	pop		bc
	ret

;==============================================================================;
; doCmd01
; Performs setup work for Command $01 (Slot Change).

doCmd01:
	di
	xor		a
	out		(0xC),a			; write 0 to port 0xC (Respond to 68K)
	out		(0),a			; write to port 0 (Clear sound code)
	ld		sp,0xFFFC		; set stack pointer

	; call Command 01
	ld		hl,command_01
	push	hl
	retn

;==============================================================================;
; doCmd03
; Performs setup work for Command $03 (Soft Reset).

doCmd03:
	di
	xor		a
	out		(0xC),a			; write 0 to port 0xC (Respond to 68K)
	out		(0),a			; write to port 0 (Clear sound code)
	ld		sp,0xFFFC		; set stack pointer

	; call Command 03
	ld		hl,command_03
	push	hl
	retn

;==============================================================================;
; SetDefaultBanks
; Sets the default program banks. This setup treats the M1 ROM as linear space.

SetDefaultBanks:
	SetBank	0x1E,8			; Set $F000-$F7FF bank to bank $1E (30 *  2K)
	SetBank	0xE,9			; Set $E000-$EFFF bank to bank $0E (14 *  4K)
	SetBank	6,0xA			; Set $C000-$DFFF bank to bank $06 ( 6 *  8K)
	SetBank	2,0xB			; Set $8000-$BFFF bank to bank $02 ( 2 * 16K)
	ret

;==============================================================================;
; fm_Silence
; Silences all FM channels.

; "However, if you're accessing the same address multiple times, you may write
; the address first and procceed to write the data register multiple times."
; - translated from YM2610 Application Manual, Section 9

fm_Silence:
	di
	push	af
	ld		a,0x28			; Slot and Key On/Off
	out		(4),a			; write to port 4 (address 1)
	rst		8				; Write delay 1 (17 cycles)
	;---------------------------------------------------;
	ld		a,0x01			; FM Channel 1
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	;---------------------------------------------------;
	ld		a,0x02			; FM Channel 2
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	;---------------------------------------------------;
	ld		a,0x05			; FM Channel 3
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	;---------------------------------------------------;
	ld		a,0x06			; FM Channel 4
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	pop		af
	ret

;==============================================================================;
; ssg_Silence
; Silences all SSG channels.

ssg_Silence:
	di
	ld		de,0x0800		; SSG Channel A Volume/Mode
	write45					; write to ports 4 and 5
	;-------------------------------------------------;
	ld		de,0x0900		; SSG Channel B Volume/Mode
	write45					; write to ports 4 and 5
	;-------------------------------------------------;
	ld		de,0x0A00		; SSG Channel C Volume/Mode
	write45					; write to ports 4 and 5
	;-------------------------------------------------;
	; Disable all SSG channels... you may not want this.
	ld		de,0x073F		; Disable all SSG channels (Tone and Noise)
	write45					; write to ports 4 and 5
	ret

;==============================================================================;
; pcma_Silence
; Silences all ADPCM-A channels.

pcma_Silence:
	; (unimplemented in this demo)
	ret

;==============================================================================;
; pcmb_Silence
; Silences the ADPCM-B channel.

pcmb_Silence:
	di
	ld		de,0x1001			; $1001		Force stop synthesis
	write45

	dec		e					; $1000		Stop ADPCM-B output
	write45
	ret

;==============================================================================;
; all of these are writes to ports 4/5:
; Set all Timer flags (0x273F)
; Clear all Timer flags (0x2700)
; Reset A/B flags, Load and Enable Timer B (0x273A)
; Reset A/B flags, Load and Enable Timer A (0x2735)
; Reset Timer A flag, Enable and Load Timers A/B (0x271F)
; Reset Timer B flag, Enable and Load Timers A/B (0x272F)

; they're also not used in this demo, so the code isn't here.

;==============================================================================;
; HandleCommand
; Handles any command that isn't already dealt with separately (e.g. $01, $03).

HandleCommand:
	ld		a,(curCommand)	; get current command
	ld		b,a

	; command $0D = stop ADPCM-B
	cp		0x0D
	jp		Z,pcmb_Silence

	; command $20 = play ADPCM-B sample
	cp		0x20
	jp		Z,play_ADPCM_B

	; command $40 = left channel output
	cp		0x40
	jp		Z,command_40
	; command $41 = right channel output
	cp		0x41
	jp		Z,command_41
	; command $42 = left/right channel output
	cp		0x42
	jp		Z,command_42

	; commands $80-$8F = change volume
	cp		0x80
	jp		Z,command_80
	cp		0x81
	jp		Z,command_81
	cp		0x82
	jp		Z,command_82
	cp		0x83
	jp		Z,command_83
	cp		0x84
	jp		Z,command_84
	cp		0x85
	jp		Z,command_85
	cp		0x86
	jp		Z,command_86
	cp		0x87
	jp		Z,command_87
	cp		0x88
	jp		Z,command_88
	cp		0x89
	jp		Z,command_89
	cp		0x8A
	jp		Z,command_8A
	cp		0x8B
	jp		Z,command_8B
	cp		0x8C
	jp		Z,command_8C
	cp		0x8D
	jp		Z,command_8D
	cp		0x8E
	jp		Z,command_8E
	cp		0x8F
	jp		Z,command_8F

	; command $C0		11.025KHz
	cp		0xC0
	jp		Z,command_C0
	; command $C1		22.050KHz
	cp		0xC1
	jp		Z,command_C1
	; command $C2		33.075KHz
	cp		0xC2
	jp		Z,command_C2
	; command $C3		44.1KHz
	cp		0xC3
	jp		Z,command_C3
	; command $C4		55.5KHz
	cp		0xC4
	jp		Z,command_C4

HandleCommand_end:
	ret

;==============================================================================;
; system command holding cell.

;------------------------------------------------------------------------------;
; command_00
; Dudley do-nothing.

command_00:
	ret

;------------------------------------------------------------------------------;
; command_01
; Handles a slot switch.

command_01:
	di
	xor		a
	out		(0xC),a			; Write 0 to port 0xC (Reply to 68K)
	out		(0),a			; Reset sound code

	call	SetDefaultBanks	; initialize banks to default config

	; (FM) turn off Left/Right, AM Sense and PM Sense
	ld		de,0xB500		; $B500: turn off for channels 1/3
	write45
	write67
	ld		de,0xB600		; $B600: turn off for channels 2/4
	write45
	write67

	; (ADPCM-A, ADPCM-B) Reset ADPCM channels
	ld		de,0x00BF		; $00BF: ADPCM-A Dump=1, all channels=1
	write67
	ld		de,0x1001		; $1001: ADPCM-B Reset=1
	write45

	; (ADPCM-A, ADPCM-B) Poke ADPCM channel flags (write 1, then 0)
	ld		de,0x1CBF		; $1CBF: Reset flags for ADPCM-A 1-6 and ADPCM-B
	write45
	ld		de,0x1C00		; $1C00: Enable flags for ADPCM-A 1-6 and ADPCM-B
	write45

	; silence FM channels
	ld		de,0x2801		; FM channel 1 (1/4)
	write45
	ld		de,0x2802		; FM channel 2 (2/4)
	write45
	ld		de,0x2805		; FM channel 5 (3/4)
	write45
	ld		de,0x2806		; FM channel 6 (4/4)
	write45

	; silence SSG channels
	ld		de,0x800		;SSG Channel A
	write45
	ld		de,0x900		;SSG Channel B
	write45
	ld		de,0xA00		;SSG Channel C
	write45

	; set up infinite loop in RAM
	ld		hl,0xFFFD
	ld		(hl),0xC3		; Set 0xFFFD = 0xC3 ($C3 is opcode for "jp")
	ld		(0xFFFE),hl	; Set 0xFFFE = 0xFFFD (making "jp $FFFD")
	ld		a,1
	out		(0xC),a			; Write 1 to port 0xC (Reply to 68K)
	jp		0xFFFD			; jump to infinite loop in RAM

;------------------------------------------------------------------------------;
; command_03
; Handles a soft reset.

command_03:
	di
	ld		a,0
	out		(0xC),a			; Write to port 0xC (Reply to 68K)
	out		(0),a			; Reset sound code
	ld		sp,0xFFFF
	jp		Start			; Go back to the top.

;------------------------------------------------------------------------------;
; command_40
; Left channel ADPCM-B output
command_40:
	xor		a
	ld		a,0x80
	ld		(intPCMB_LeftRight),a
	ld		d,0x11
	ld		a,(intPCMB_LeftRight)
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_41
; Right channel ADPCM-B output
command_41:
	xor		a
	ld		a,0x40
	ld		(intPCMB_LeftRight),a
	ld		d,0x11
	ld		a,(intPCMB_LeftRight)
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_42
; Left and Right channel ADPCM-B output
command_42:
	xor		a
	ld		a,0xC0
	ld		(intPCMB_LeftRight),a
	ld		d,0x11
	ld		a,(intPCMB_LeftRight)
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; yes these are all inefficient. this is demo code, what do you want?

; command_80 - volume $00
command_80:
	; set engine's volume
	ld		a,0x00
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_81 - volume $10
command_81:
	; set engine's volume
	ld		a,0x10
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_82 - volume $20
command_82:
	; set engine's volume
	ld		a,0x20
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_83 - volume $30
command_83:
	; set engine's volume
	ld		a,0x30
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_84 - volume $40
command_84:
	; set engine's volume
	ld		a,0x40
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_85 - volume $50
command_85:
	; set engine's volume
	ld		a,0x50
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_86 - volume $60
command_86:
	; set engine's volume
	ld		a,0x60
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_87 - volume $70
command_87:
	; set engine's volume
	ld		a,0x70
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_88 - volume $80
command_88:
	; set engine's volume
	ld		a,0x80
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_89 - volume $90
command_89:
	; set engine's volume
	ld		a,0x90
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_8A - volume $A0
command_8A:
	; set engine's volume
	ld		a,0xA0
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_8B - volume $B0
command_8B:
	; set engine's volume
	ld		a,0xB0
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_8C - volume $C0
command_8C:
	; set engine's volume
	ld		a,0xC0
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_8D - volume $D0
command_8D:
	; set engine's volume
	ld		a,0xD0
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_8E - volume $E0
command_8E:
	; set engine's volume
	ld		a,0xE0
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_8F - volume $FF
command_8F:
	; set engine's volume
	ld		a,0xFF
	ld		(intPCMB_Volume),a

	; change volume now
	ld		d,0x1B
	ld		e,a
	write45
	ret

;------------------------------------------------------------------------------;
; command_C0 - sampling rate 11025Hz (Delta-N 13019/$32DB)
command_C0:
	ld		a,0xDB
	ld		(intPCMB_SampRateL),a
	ld		a,0x32
	ld		(intPCMB_SampRateH),a

	; Delta-N Sampling Rate LSB
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	; Delta-N Sampling Rate MSB
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	ret

;------------------------------------------------------------------------------;
; command_C1 - sampling rate 22050Hz (Delta-N 26037/$65B5)
command_C1:
	ld		a,0xB5
	ld		(intPCMB_SampRateL),a
	ld		a,0x65
	ld		(intPCMB_SampRateH),a

	; Delta-N Sampling Rate LSB
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	; Delta-N Sampling Rate MSB
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	ret

;------------------------------------------------------------------------------;
; command_C2 - sampling rate 33075Hz (Delta-N 39056/$9890)
command_C2:
	ld		a,0x90
	ld		(intPCMB_SampRateL),a
	ld		a,0x98
	ld		(intPCMB_SampRateH),a

	; Delta-N Sampling Rate LSB
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	; Delta-N Sampling Rate MSB
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	ret

;------------------------------------------------------------------------------;
; command_C3 - sampling rate 44100Hz (Delta-N 52075/$CB6B)
command_C3:
	ld		a,0x6B
	ld		(intPCMB_SampRateL),a
	ld		a,0xCB
	ld		(intPCMB_SampRateH),a

	; Delta-N Sampling Rate LSB
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	; Delta-N Sampling Rate MSB
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	ret

;------------------------------------------------------------------------------;
; command_C4 - sampling rate 55500Hz (Delta-N 65536/$FFFF)
command_C4:
	ld		a,0xFF
	ld		(intPCMB_SampRateL),a
	ld		(intPCMB_SampRateH),a

	; Delta-N Sampling Rate LSB
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	; Delta-N Sampling Rate MSB
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	ret

;==============================================================================;
; play_ADPCM_B
; Play ADPCM-B sample

; XXX: Sample addresses are hardcoded.
; A few other values depend on driver calls for changes.

; This is probably not fully recommended in a production driver; it's just an
; example of a "working" playback routine.

play_ADPCM_B:
	di
	; $1C80		Mask Flag B (reset flags)
	ld		de,0x1C80
	write45

	; $1C00		Enable Flags
	ld		de,0x1C00
	write45

	; Start/Repeat/Reset ($10 on ports 4/5)
	ld		de,0x1000
	write45

	; Left/Right Output ($11 on ports 4/5)
	ld		d,0x11
	ld		a,(intPCMB_LeftRight)
	ld		e,a
	write45

	; start address/256 LSB ($12 on ports 4/5)
	ld		de,0x1200
	write45
	; start address/256 MSB ($13 on ports 4/5)
	ld		de,0x1300
	write45

	; end address/256 LSB ($14 on ports 4/5)
	ld		de,0x14FE
	write45
	; end address/256 MSB ($15 on ports 4/5)
	ld		de,0x1500
	write45

	; Delta-N Sampling Rate LSB ($19 on ports 4/5)
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	; Delta-N Sampling Rate MSB ($1A on ports 4/5)
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	; Channel Volume ($1B on ports 4/5)
	ld		d,0x1B
	ld		a,(intPCMB_Volume)
	ld		e,a
	write45

	; start output
	ld		de,0x1080		; start, no repeat
	write45

	; Output control
	; $1080		Start (no repeat)
	; $1090		Start (with repeat)
	; $1080		Cancel repeat
	; $1001		Force stop synthesis
	; $1000		Stop ADPCM-B output
	ret

;==============================================================================;
; ADPCM-B Sample Data
; format:
; 2 words - Start and End address/256
; 2 words - Delta-N sampling rates

;------------------------------------------------------------------------------;
; Calculating playback rate ("samprate" in samples_PCMB table):

; PR = playback rate
; freq = sampling rate

;      65536 * freq
; PR = ------------ (round result to nearest whole number)
;          55.5

; example: 44.1KHz
; (65536*44.1)/55.5 = 52,074.5513513514 -> 52,075 (0xCB6B)
;------------------------------------------------------------------------------;

; this is currently not used because I'm doing hardcoded crap.
; sorry! I suck at z80 -freem
samples_PCMB:
	;word	startaddr,endaddr,samprate
	word	0x0000,0x00FE,0xCB6B

;==============================================================================;
; RAM defines at $F800-$FFFF
	include "soundram.inc"
